#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling Mount event"""

from ionas.request import Entry, EntryList
from iobase.xmldata import XmlConfNode
from iobase.exception import ExecError, NotFoundError
import libevh.evhcmd as evhcmd
import libevh.util as util
import fal_msginfo as msginfo
import fal_info
from product_conf.reqinfo import (Results, ReqTypes)
from evh_config import VOL_INFO

NAME_PATH = "eventinfo/volume/name"
SHARE_LIST_PATH = "sharelist/share/replication[@enable='true']/.."
PATH_NAME = "path"
SHARES_NODE_FMT = "<shares>%s</shares>"


class EvhMount(evhcmd.EvhCmd):
    """Mount Event Handler command implementation class."""

    #
    # Utility method
    #

    def __send_request(self, share_node_list):

        if not share_node_list:
            return

        entrylist = EntryList()

        for share_node in share_node_list:
            entrylist.append_entry(Entry(ReqTypes.SET,
                XmlConfNode(SHARES_NODE_FMT % str(share_node))))

        res_entrylist = self.send_entrylist(entrylist,
            fal_info.FAL_REPLICATION_PATH)

        for res_entry in res_entrylist.get_entry_list():
            if res_entry.get_result() != Results.OK:
                raise ExecError(fal_info.FAL_REPLICATION_PATH)

    def __get_share_node_list(self, mnt_point):

        share_config = self.send_entry(Entry(ReqTypes.LOAD),
            fal_info.FAL_SHARES_PATH).get()

        share_node_list = []
        for share_node in share_config.get_node_list(SHARE_LIST_PATH):
            if share_node.get_node(PATH_NAME).get_text().startswith(mnt_point):
                share_node_list += [share_node]

        if not share_node_list:
            raise NotFoundError

        return share_node_list

    def __set_replication(self, body):

        vol_name = body.get_node(NAME_PATH).get_text()

        if not util.is_share_mode_by_vol_name(vol_name):
            return

        mnt_point = VOL_INFO[vol_name]["mnt_point"]

        try:
            share_node_list = self.__get_share_node_list(mnt_point)
        except NotFoundError:
            return

        self.__send_request(share_node_list)
        util.save_event_log(msginfo.Msg.MSG_REPLICATION_START)

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        self.__set_replication(entry.get())

        entry.set_result(Results.OK)
        return entry

    def __init__(self):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, evhcmd.EvhCmdOption())

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhMount().handle_request()
